#!/usr/bin/perl -w

# RN Luben October 2011  v.006

use Getopt::Long;
use Text::ParseWords;

# Get program arguments
GetOptions( 
   "b" => sub { $backup = 1 },
   "backup" => sub { $backup = 1 },
   "s" => sub { $nheaders = 1 },
   "single" => sub { $nheaders = 1 },
   "d" => sub { $nheaders = 2 },
   "double" => sub { $nheaders = 2 },
   "p" => sub { $byname = -1 },
   "byposition" => sub { $byname = -1 },
   "n" => sub { $byname = 1 },
   "byname" => sub { $byname= 1 },
   "verbose" => sub { $verbose= 1 },
   "l=s" => \$lookup,
   "lookup=s" => \$lookup,
   "i=s" => \$infile,
   "infile=s" => \$infile,
   "o=s" => \$outfile,
   "outfile=s" => \$outfile,
   "c=s" => \$column,
   "column=s" => \$column,
   "sep=s" => \$sep,
   "outsep=s" => \$outsep,
   "predrop=s" => \$predrop,
   "h" => sub { $usage = 1 },
   "help" => sub { $usage = 1 },
   "usage" => sub { $usage = 1 } 
);

# Initialize variables and make checks
$usage = $usage||0;
$verbose=$verbose||0;
print STDERR "\n\n" if $verbose==1;
$backup=$backup||0;
$nheaders=$nheaders||1;
$column=$column||"";
$predrop=$predrop||0;
$sep=$sep||"";
$outsep=$outsep||"\t";
$column="1,2,3" if $column eq "";
($i,$j,$k) = split(",",$column);
# Note that numbering starts at 1 (not zero).  If column I or any other is given as 0 this will be re-assigned the default
# along with missing I, J or K.
$i=$i||1;
$j=$j||2;
$k=$k||3;
$byname=$byname||1;
$outfile1=$outfile||$$;
$outfile=$outfile||"";
$infile=$infile||"";
print STDERR "NOTE: Output file was not given - so writing back to the input file\n" if $outfile eq "" and  $usage ==0 and $verbose == 1;
$lookup=$lookup||"";
print STDERR "ERROR: Lookup file was not given.  Cannot continue\n" if $lookup eq "" and  $usage ==0;
die if $lookup eq ""  and  $usage ==0;
print STDERR "ERROR: Lookup file not found. Cannot continue\n" if not(-e "$lookup") and  $usage ==0;
die if not(-e "$lookup")  and  $usage ==0;
print STDERR "ERROR: Input file was not given.  Cannot continue\n" if $infile eq "" and  $usage ==0;
die if $infile eq ""  and  $usage ==0;
print STDERR "ERROR: Input file not found. Cannot continue\n" if not(-e "$infile") and  $usage ==0;
die if not(-e "$infile")  and  $usage ==0;
die "ERROR:  Sorry, but byposition is not available.  Cannot continue\n"  if $byname==-1;


if($verbose==1) {
   print STDERR "NOTE: Arguments given:\n\tSingle or Double header needed in the outfile: $nheaders\n\tLink to the lookup file made by name (1) or by position (0): $byname\n\tOptional output file name: $outfile, $outfile1\n\tName of the text file used for cross-referencing the headers: $lookup\n\tHas usage been requested?: $usage\n\tCreate a backup file 1=true 0=false: $backup\n\tSeparator used in lookup file: $sep\n\tDrop a number of characters from the start of the old header before matching: $predrop\n\n";
}

# Output the usage and finish if requested
die << "END" if $usage =~ /1/;
NAME
     headerswitch.pl - replace the header of a text file with another by referencing a lookup file.

SYNOPSIS
     headerswitch.pl [OPTION]...

DESCRIPTION
     Replace the first line of a text file containing headings
     with an alternative set of headings where the two sets are 
     linked in an intermediate lookup file.

     -b, --backup
          Keep a copy of the input file before making changed
          with the extension .orig
    
     -c, --column=I,J (single) or =I,J,K (double)
          Normally, the lookup file will be arranged so that
          the first column will contain the existing header,
          and the second contains the replacement header. If
          a double header is required this normally appears 
          in the third column. This is also the case when 
          linking is done by position - the first column being 
          ignored. However, the columns may be given explicitly
          if necessary. The I parameter should be given as 0
          when linking by position.

     -d, --double
          Replace with a two line header

     -h, --help
          Help and usage 

     -i, --infile=FILE
          The name of the input file (required.)

     -l, --lookup=FILE 
          The name of the lookup file (required.)

     -n, --byname
          Headings are matched using their name, ignoring
          case. (default) 
          
     -o, --outfile=FILE
          The name of the optional output file. When not given
          the changes are written back to the input file. 

     --outsep=CHAR
          Use CHAR as a separator in the output file instead 
          of tab

     -p, --byposition
          Rather than matching the headings by name, their 
          position is used to find an alternative header in 
          the lookup

     --predrop=NUM
          Drop NUM characters from each input file header prior 
          to linking

     --sep=CHAR
          Use CHAR as a separator in the lookup file instead of 
          whitespace

     -s, --single
          Replace with a single line header (default)

     -u, --usage
          Help and usage 

     --verbose
          Extra information given

EXAMPLES

     headerswitch.pl --infile=test1.txt --outfile=out.txt --lookup=lookup1.txt --verbose

     headerswitch.pl --infile=in.txt --outfile=out.txt --lookup=lookup.txt 

     headerswitch.pl --column=1,4 --infile=in.txt --outfile=out.txt --backup --lookup=lookup.txt 

     headerswitch.pl --infile=in.txt --byposition --column=0,2,1 --double --lookup=lookup.txt 

     headerswitch.pl --infile=test1.txt --outfile=out.txt --lookup=nutrients.csv --verbose --sep=, --double

     headerswitch.pl --infile=test1.txt --backup --lookup=nutrients.csv --sep=, --predrop=9 --double

ABOUT

     v.006 October 2011
          
END

# Read in the lookup file
open(LOOKUP, "< $lookup") or die "can't open $lookup: $!";
while(<LOOKUP>) {
   chomp;
   @words = quotewords('\s+', 0, $_) if $sep eq "";
   @words = quotewords($sep, 0, $_) if $sep ne "";
   $header_i = $words[$i - 1];
   $header_j = $words[$j - 1];
   $header_k = $words[$k - 1];
   $header_k=$header_k||"";
   print STDERR "NOTE: Header I is $header_i, header J is $header_j and header K is $header_k\n" if $verbose==1; 
   $link1{$header_i} = $header_j;
   $link2{$header_i} = $header_k;
   @words=();
}



open(IN, "< $infile") or die "can't open $infile: $!"; 
open(OUT, "> $outfile1") or die "can't open $outfile1: $!"; 

$line=1;
while (<IN>) { 
   if( $line==1 ) {
      @inheaders = split('\s+',$_);
      foreach $inheader (@inheaders) {
         $inheader = substr($inheader,$predrop,length($inheader)-$predrop);
         print OUT $outsep unless $line==1;
         $line++;;
         print OUT $link1{$inheader} or die "can't write $outfile: $!";
         print OUT ' ('.$link2{$inheader}.')' if($nheaders==1);
      }
      print OUT "\n" if $nheaders==2;
      if( $nheaders==2 ) {
         $line=1;
         foreach $inheader (@inheaders) {
            print OUT $outsep unless $line==1;
            $line++;
            print OUT $link2{$inheader} or die "can't write $outfile: $!";
         }
      }
      print OUT "\n" or die "can't write $outfile: $!";
      $line=0;
   } else {
      @indata = split('\s+',$_);
      print OUT join($outsep,@indata)."\n" or die "can't write $outfile: $!"; 
   }
}

close(IN) or die "can't close $infile: $!"; 
close(OUT) or die "can't close $outfile: $!"; 

if($backup ==1) {
   rename($infile, "$infile.orig") or die "can't rename $infile to $infile.orig: $!"; 
}
if($outfile eq "") {
   rename($outfile1, $infile) or die "can't rename $outfile to $infile: $!";
}
